@@ -37,10 +37,10 @@
 #'exp <- NULL
 #'exp$data <- array(rnorm(45), dim = c(member = 7, ftime = 8))
 #'class(exp) <- 's2dv_cube'
-#'exp$Dates$start <- c(seq(as.Date("01-07-1993", "%d-%m-%Y", tz = 'UTC'),
-#'                    as.Date("01-08-1993","%d-%m-%Y", tz = 'UTC'), "day"),
-#'                seq(as.Date("01-07-1994", "%d-%m-%Y", tz = 'UTC'),
-#'                    as.Date("01-08-1994","%d-%m-%Y", tz = 'UTC'), "day"))
+#'exp$attrs$Dates <- c(seq(as.Date("01-07-1993", "%d-%m-%Y", tz = 'UTC'),
+#'                         as.Date("01-08-1993","%d-%m-%Y", tz = 'UTC'), "day"),
+#'                     seq(as.Date("01-07-1994", "%d-%m-%Y", tz = 'UTC'),
+#'                         as.Date("01-08-1994","%d-%m-%Y", tz = 'UTC'), "day"))
 #'SA <- CST_PeriodMean(exp)
 #'
 #'@import multiApply
@@ -48,32 +48,28 @@
 CST_PeriodMean <- function(data, start = NULL, end = NULL,
                            time_dim = 'ftime', na.rm = FALSE,
                            ncores = NULL) {
-# Consider to add an option for providing tx and tn in data
+   # Check 's2dv_cube'
   if (!inherits(data, 's2dv_cube')) {
-    stop("Parameter 'data' must be of the class 's2dv_cube', ",
-         "as output by CSTools::CST_Load.")
+    stop("Parameter 'data' must be of the class 's2dv_cube'.")
   }
-  # when subsetting is needed, dimensions are also needed:
+  # Dates subset
   if (!is.null(start) && !is.null(end)) {
-    if (is.null(dim(data$Dates$start))) {
-      if (length(data$Dates$start) != dim(data$data)[time_dim]) {
-        if (length(data$Dates$start) ==
-            prod(dim(data$data)[time_dim] * dim(data$data)['sdate'])) {
-          dim(data$Dates$start) <- c(dim(data$data)[time_dim],
-                                     dim(data$data)['sdate'])
-        } else {
-          warning("Dimensions in 'data' element 'Dates$start' are missed/unmatched. All data would be used.")
-        }
-      }
+    if (is.null(dim(data$attrs$Dates))) {
+      warning("Dimensions in 'data' element 'attrs$Dates' are missed and ",
+              "all data would be used.")
+      start <- NULL
+      end <- NULL
     }
   }
-  total <- PeriodMean(data = data$data, dates = data$Dates[[1]], start, end,
+
+  total <- PeriodMean(data = data$data, dates = data$attrs$Dates, start, end,
                       time_dim = time_dim, na.rm = na.rm, ncores = ncores)
   data$data <- total
   if (!is.null(start) && !is.null(end)) {
-     data$Dates <- SelectPeriodOnDates(dates = data$Dates[[1]],
-                                start = start, end = end,
-                                time_dim = time_dim, ncores = ncores)
+     data$attrs$Dates <- SelectPeriodOnDates(dates = data$attrs$Dates,
+                                             start = start, end = end,
+                                             time_dim = time_dim, 
+                                             ncores = ncores)
   }
   return(data)
 }
